/*-
 * Copyright (C)2005..2024 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#)Copyright (C)2005..2024 @BABOLO http://www.babolo.ru/"
#ident "$Id: create.c,v 1.13 2024/04/14 20:41:05 babolo Exp $"

#define BLIN_COMPAT  4
#define MULAR_COMPAT VMAJOR
#define MULAR_INTERNAL

#include <sys/types.h>
#include <sysexits.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include "multilar.h"

BLIN_flag mular_default;

mular_descriptor *
mular_create(BLIN_flag flags, int lemax, size_t unit, const size_t *size) {
    mular_descriptor *mular;
    size_t mds;

#   define blin_internal_flags (flags & BLIN_MASK)
    ifBLIN_QX2("+ flags=%08X lemax=%d unit=%"BLIN_U, flags, lemax, unit);
    if  (lemax < 1 || lemax >= 31) {
        ifBLIN_QW0("lemax: %d", lemax);
        errno = EINVAL;
        return(NULL);
    }
    mds = ((size_t)lemax - 1) * sizeof(size_t) + sizeof(mular_descriptor);
    if  (!(mular = malloc(mds))) {
        ifBLIN_QW0("No mem");
        return(NULL);
    }
    bzero(mular, mds);
    mular->flags = flags;
    mular_default = flags;
    mular->lemax = (u_int16_t)lemax;
    if  (!!unit) {
        mular->unit = unit;
    } else {
        mular->unit = sizeof(void*);
        mular->flags |= MULAR_PFRE | MULAR_SWP1 | MULAR_SWP2;
    }
    if  (mular->flags & (MULAR_CHAR | MULAR_PDMP | MULAR_PFRE) && mular->unit < sizeof(void*)) {
        ifBLIN_QX0("No space for pointer in unit");
        errno = EINVAL;
        return(NULL);
    }
    if  (!!size) memmove(&mular->size, size, (size_t)lemax * sizeof(size_t));
    ifBLIN_QX2( "- flags=%08X lemax=%d unit=%"BLIN_U" size=%"BLIN_U
              , mular->flags, mular->lemax, mular->unit, mds
              );
    return(mular);
#   undef blin_internal_flags
}
